import { defineComponent, ref, computed, onMounted, markRaw, watch, nextTick, createVNode } from 'vue';
import CodeMirror from 'codemirror';
import { getUsingConfig, isJson } from '@ltdp/utils/util';
import { generateProps, generateEmits } from '@d-render/shared';
import { componentScheme } from './component.scheme';
import 'codemirror/mode/clike/clike.js';
import 'codemirror/addon/hint/show-hint';
import 'codemirror/mode/javascript/javascript';
import 'codemirror/mode/xml/xml';
import 'codemirror/addon/hint/sql-hint';
import 'codemirror/addon/hint/javascript-hint';
import 'codemirror/addon/edit/closebrackets';
import 'codemirror/lib/codemirror.css';
import 'codemirror/addon/hint/show-hint.css';
import 'codemirror/theme/dracula.css';
import 'codemirror/theme/3024-day.css';

var index = defineComponent({
  name: "CipCodeMirror",
  props: generateProps(componentScheme),
  emit: generateEmits(componentScheme),
  setup(props, {
    expose,
    emit
  }) {
    const codemirror = ref();
    const codemirror$ = ref();
    const modeBridge = computed(() => {
      return getUsingConfig(props.mode, props.type === "json" ? "application/json" : `text/${props.type}`);
    });
    const formatterValue = (val) => {
      var _a;
      if (modeBridge.value === "application/json" && val) {
        let data = val;
        if (typeof data === "string" && isJson(data)) {
          data = JSON.parse(data);
        }
        return (_a = JSON.stringify(data, null, 2)) != null ? _a : "";
      } else {
        return val;
      }
    };
    const getCodeMirrorInstance = () => {
      var _a, _b;
      const instance = CodeMirror.fromTextArea(codemirror$.value, {
        value: formatterValue(props.modelValue),
        mode: modeBridge.value,
        theme: props.theme,
        indentUnit: props.indentUnit,
        lineNumbers: props.lineNumbers,
        lineWrapping: props.lineWrapping,
        readOnly: props.readonly,
        extraKeys: {}
      });
      instance.setSize((_a = props.width) != null ? _a : "100%", (_b = props.height) != null ? _b : "auto");
      instance.on("change", (instance2, obj) => {
        const {
          origin
        } = obj;
        const value = instance2.getValue();
        codemirror.value.save();
        if (origin !== "setValue") {
          emit("update:modelValue", value);
        }
      });
      instance.on("inputRead", function() {
        const defaultHintOptions = {
          completeSingle: false
        };
        const hintOptions = props.customHintOption ? props.customHintOption(CodeMirror) : defaultHintOptions;
        instance.showHint(hintOptions);
      });
      return instance;
    };
    onMounted(() => {
      codemirror.value = markRaw(getCodeMirrorInstance());
      watch(() => props.modelValue, (val) => {
        var _a, _b, _c;
        const codeMirrorValue = (_b = (_a = codemirror.value) == null ? void 0 : _a.getValue) == null ? void 0 : _b.call(_a);
        if (codeMirrorValue === val)
          return;
        codemirror.value.setValue((_c = formatterValue(val)) != null ? _c : "");
        nextTick().then(() => {
          codemirror.value.refresh();
        });
      }, {
        immediate: true
      });
    });
    const optionMap = {
      readonly: "readOnly",
      indentUnit: "indentUnit",
      lineNumbers: "lineNumbers",
      lineWrapping: "lineWrapping"
    };
    const needWatchProps = ["readonly", "indentUnit", "lineNumbers", "lineWrapping"];
    watch(() => needWatchProps.map((key) => props[key]), (valueList, oldValueList) => {
      valueList.forEach((v, index) => {
        if (v !== oldValueList[index]) {
          const key = optionMap[needWatchProps[index]];
          codemirror.value.setOption(key, v);
        }
      });
    });
    expose({
      codemirror
    });
    return () => createVNode("div", null, [createVNode("textarea", {
      "ref": codemirror$
    }, null)]);
  }
});

export { index as default };
